(***********************************************)
(*      (c) Copyright 1987 by Jens Kalski      *)
(***********************************************)
(* Programmmodul  : DM-COPY.INC / (26.08.1988) *)
(* Sprache        : TURBO-PASCAL 3.0           *)
(* Computer       : Schneider CPC464           *)
(* Betriebssystem : CP/M 2.2                   *)
(***********************************************)

FUNCTION Kopie:BOOLEAN;

CONST
  Quelle = TRUE;
  Ziel   = FALSE;

TYPE
  VerwalterT = RECORD                                      (* Verwalter-Typ *)
                 Nummer,                                (* Nummer der Datei *)
                 Von,                            (* VON Record im Puffer an *)
                 Bis         : INTEGER;            (* BIS zu welchem Puffer *)
                 DateiAnfang : BOOLEAN;               (* Anfang der Datei ? *)
               END;

VAR
  Flag,                   (* Anzeiger fuer ordnungsgemaessen Programmablauf *)
  EndFlag,                                          (* Dateiende erreicht ? *)
  RestFlag,                                  (* Noch ein Rest zu kopieren ? *)
  Lwk1Frage,                                   (* Quell-Laufwerk abfragen ? *)
  Lwk2Frage,                                    (* Ziel-Laufwerk abfragen ? *)
  GleichFlag,                              (* Quell- gleich Ziel-Laufwerk ? *)
  AnfangFlag      : BOOLEAN;                    (* Zeigt den Dateianfang an *)

  i,                                                        (* Laufvariable *)
  PufferRecord,       (* Pufferrecord in den als naechstes geschrieben wird *)
  LeseRecord,             (* Dateirecord von dem als naechstes gelesen wird *)
  SchreibRecord,      (* Dateirecord auf den als naechstes geschrieben wird *)
  VerwalterAnzahl : INTEGER;          (* Anzahl der gespeicherten Verwalter *)
  DateiName       : Str12;  (* Hilfsspeicher fuer den jeweiligen Dateinamen *)
  Verwalter       : ARRAY(.0..MaxDat.) OF VerwalterT;     (* Verwalter-Feld *)

  (* Fragt nur nach einer Diskette fuer das Quell-Laufwerk, *)
  (* wenn es auch ein Diskettenlaufwerk ist. Hier wird nur  *)
  (* das Laufwerk gewechselt, ohne ein DISK-RESET.          *)
  PROCEDURE QuellFrage;
  VAR c : CHAR;
  BEGIN
    IF NormLwk(Lwk1) THEN BEGIN    (* Wenn Quell-Laufwerk = Diskettenlwk. ? *)
      DialogZeile('Bitte Quelldiskette in Laufwerk '+FormLwk(Lwk1)+
                  ' einlegen. (ENTER)',Nein);
      c:=LiesZeichen((.^M.));
      LoeschDialogZeile;
    END;
    SetzeLaufwerk(Lwk1,Nein);
  END;

  (* Gibt Ein- Ausgabefehlermeldungen aus *)
  PROCEDURE Fehler(DateiName:Str12;EinAusFlag:BOOLEAN;Nummer:INTEGER);
  CONST
    Fehler      : ARRAY(.0..4.) OF STRING(.11.) = ('geoeffnet',
                  'adressiert','gelesen','erweitert','geschlossen');
    SchreibLese : ARRAY(.BOOLEAN.) OF STRING(.7.) = ('SCHREIB','LESE');
  VAR
    c : CHAR;
  BEGIN
    DialogZeile(SchreibLese(.EinAusFlag.) + 'FEHLER : ' + DateiName
                + ' kann nicht ' + Fehler(.Nummer.)
                + ' werden. (ENTER=Weiter)',Nein);
    c:=LiesZeichen((.^M.));
    LoeschDialogZeile;
  END;

  FUNCTION DateiOeffnen(DateiNummer,Lwk:INTEGER; QuellZiel,
                        LwkWechselFlag,NachfrageFlag:BOOLEAN):BOOLEAN;

    (* Fragt nur nach einer Diskette fuer das Ziel-Laufwerk *)
    (* Laufwerk, wenn es auch ein Diskettenlaufwerk ist.    *)
    (* Hier wird das Laufwerk nur mit einem DISK-RESET      *)
    (* gewechselt, wenn es ein Diskettenlaufwerk ist.       *)
    PROCEDURE ZielFrage;
    VAR c : CHAR;
    BEGIN
      IF NormLwk(Lwk2) THEN BEGIN
        DialogZeile('Bitte Zieldiskette in Laufwerk '+FormLwk(Lwk2)+
                    ' einlegen, Schreibschutz beachten. (ENTER)',Nein);
        c:=LiesZeichen((.^M.));
        LoeschDialogZeile;
      END;
      SetzeLaufwerk(Lwk2,NormLwk(Lwk2));
    END;

  BEGIN
    DateiName:=FormName(DateiNummer);  (* Dateinamen fuer ASSIGN herstellen *)
    IF LwkWechselFlag              (* Soll das Laufwerk gewechselt werden ? *)
      THEN IF NachfrageFlag   (* JA -> Soll eine Benutzerabfrage erfolgen ? *)
        THEN IF QuellZiel              (* JA -> Quell- oder Ziel-Laufwerk ? *)
          THEN QuellFrage                        (* Quell-Laufwerk anmelden *)
          ELSE ZielFrage                          (* Ziel-Laufwerk anmelden *)
        ELSE SetzeLaufwerk(Lwk,NormLwk(Lwk));      (* Anmelden ohne Abfrage *)
    Assign(Datei,DateiName);                              (* Datei anmelden *)
    DateiOeffnen:=OeffneDatei;                             (* Datei oeffnen *)
  END;

  FUNCTION SpringZuRecord(Rec:INTEGER):BOOLEAN;
  BEGIN
    (*$I-*) Seek(Datei,Rec); (*$I+*)
    SpringZuRecord:=IO_Flag;
  END;

  FUNCTION SchliesseDatei:BOOLEAN;
  BEGIN
    (*$I-*) Close(Datei); (*$I+*)
    SchliesseDatei:=IO_Flag;
  END;

  (* Die Funktion leert den Record-Puffer entsprechend den, *)
  (* im VERWALTER gespeicherten Dateinummern und -groessen. *)
  FUNCTION LeerePuffer:BOOLEAN;
  VAR
    Flag,                 (* Anzeiger fuer ordnungsgemaessen Programmablauf *)
    Abbruch,     (* Wird gesetzt, wenn die Funktion abgebrochen werden soll *)
    WechselFlag : BOOLEAN;             (* Soll Diskette gewechselt werden ? *)
    i,                                                     (* Hilfsvariable *)
    AnzahlSchreiben,                               (* Zu schreibende Record *)
    AnzahlGeschrieben : INTEGER;            (* Wirklich geschriebene Record *)

    FUNCTION DateiLoeschenOeffnen:BOOLEAN;
    BEGIN
      (*$I-*) ReWrite(Datei); (*$I+*)
      DateiLoeschenOeffnen:=IO_Flag;
    END;

  BEGIN
    i:=0; Flag:=Ja; WechselFlag:=Ja;            (* Variablen initialisieren *)

    (* Solange noch Verwalter da sind und kein Fehler vorliegt *)
    WHILE (i<=VerwalterAnzahl) AND Flag DO
      WITH Verwalter(.i.) DO BEGIN
        Abbruch:=Nein;
        Flag:=DateiOeffnen(Nummer,Lwk2,Ziel,WechselFlag,Lwk2Frage);
        WechselFlag:=Nein;         (* Nur bei erstem Puffereintrag auf TRUE *)
        Lwk2Frage:=Nein;           (* Nur bei erstem Puffereintrag auf TRUE *)
        DialogZeile('Puffer --> '+DateiName,Nein);

        IF Flag
          THEN IF DateiAnfang      (* Datei schon vorhanden / Dateianfang ? *)
            THEN BEGIN                                       (* Dateianfang *)
              SchreibRecord:=0;
              (*$I-*) Close(Datei); (*$I+*)             (* Datei schliessen *)
              DialogZeile(DateiName+' ueberschreiben ?',Ja);
              IF JaNein(Ja)
                THEN BEGIN
                  Flag:=DateiLoeschenOeffnen;       (* Datei ueberschreiben *)
                  DialogZeile('Puffer --> '+DateiName,Nein);
                END
                ELSE BEGIN                  (* Kopie dieser Datei abbrechen *)
                  Flag:=Ja;                  (* Aeussere Schleife Abbrechen *)
                  RestFlag:=Nein;                         (* Kein Dateirest *)
                  EndFlag:=Ja;                        (* Dateiende erreicht *)
                  Abbruch:=Ja;              (* Kopie dieser Datei abbrechen *)
                END;
            END
            ELSE BEGIN                                  (* Kein Dateianfang *)
              Flag:=SpringZuRecord(SchreibRecord);
              IF NOT Flag THEN Fehler(DateiName,Aus,3);
            END

          ELSE IF DateiAnfang (* Datei noch nicht vorhanden / Dateianfang ? *)
            THEN BEGIN                                       (* Dateianfang *)
              SchreibRecord:=0;
              Flag:=DateiLoeschenOeffnen;            (* Datei neu eroeffnen *)
              IF NOT Flag THEN Fehler(DateiName,Aus,0);
            END
            ELSE BEGIN                                  (* Kein Dateianfang *)
              Flag:=Nein;
              Fehler(DateiName,Aus,3);
            END;

        IF Flag THEN BEGIN                    (* Dateioeffnen erfolgreich ? *)

          (* Naechsten Lese-Record fuer diese Datei berechnen *)
          LeseRecord:= (LeseRecord + Succ(Bis-Von)) *
                       Ord((i=VerwalterAnzahl) AND RestFlag);

          IF NOT Abbruch THEN BEGIN
            AnzahlSchreiben:=Succ(Bis-Von);      (* Ausgabeanzahl berechnen *)
                     (* Naechsten Schreib-Record fuer diese Datei berechnen *)
            SchreibRecord:=SchreibRecord+AnzahlSchreiben;
            (*$I-*)                       (* Records in die Datei schreiben *)
            BlockWrite(Datei,Puffer^(.Von.),AnzahlSchreiben,
                       AnzahlGeschrieben);
            (*$I+*)
            Flag:=(IO_Flag AND (AnzahlSchreiben=AnzahlGeschrieben));
            IF Flag
              THEN BEGIN                           (* Schreiben erfolgreich *)
                Flag:=SchliesseDatei;
                IF NOT Flag THEN Fehler(DateiName,Aus,4);
              END
              ELSE BEGIN
                Fehler(DateiName,Aus,3);  (* Schreiben nicht erfolgreich *)
                Flag:=Nein;
                EndFlag:=Ja;
                RestFlag:=Nein;
              END;
          END;
        END;
        i:=Succ(i);                                  (* Naechsten Verwalter *)
      END;

    PufferRecord:=0;            (* Zeiger wieder auf den Anfang des Puffers *)
    LeerePuffer:=Flag;
    IF GleichFlag THEN Lwk2Frage:=Ja;
  END;

  FUNCTION FuellePuffer(Nr:INTEGER):BOOLEAN;
  VAR
    Flag,                 (* Anzeiger fuer ordnungsgemaessen Programmablauf *)
    WechselFlag   : BOOLEAN;           (* Soll Diskette gewechselt werden ? *)
    AnzahlLesen,                          (* Anzahl der zu lesenden Records *)
    AnzahlGelesen : INTEGER;       (* Anzahl der wirklich gelesenen Records *)

  BEGIN
    Flag:=Ja;                                       (* Variablen vorbelegen *)
    EndFlag:=Nein;
    AnfangFlag:=Ja;
    WechselFlag:=Ja;
    VerwalterAnzahl:=Succ(VerwalterAnzahl);

    WHILE NOT EndFlag AND Flag DO (* Solange noch Records in der Datei sind *)
      WITH Verwalter(.VerwalterAnzahl.) DO BEGIN
        Flag:=DateiOeffnen(Nr,Lwk1,Quelle,WechselFlag,Lwk1Frage);
        WechselFlag:=Nein;         (* Nur bei erstem Puffereintrag auf TRUE *)
        Lwk1Frage:=Nein;           (* Nur bei erstem Puffereintrag auf TRUE *)
        DialogZeile(DateiName+' --> Puffer',Nein);
        IF Flag
          THEN BEGIN                           (* Dateioeffnung erfolgreich *)

            (* Mitten aus einer Datei heraus lesen *)
            IF RestFlag THEN Flag:=SpringZuRecord(LeseRecord);

            IF Flag
              THEN BEGIN   (* Dateioeffnung und -positionierung erfolgreich *)

                (* Restpuffer berechnen *)
                AnzahlLesen:=Succ(PufferGroesse-PufferRecord);

                (*$I-*)                                    (* Records lesen *)
                BlockRead(Datei,Puffer^(.PufferRecord.),AnzahlLesen,
                          AnzahlGelesen);
                (*$I+*)

                Flag:=IO_Flag;
                IF Flag
                  THEN BEGIN                           (* Lesen erfolgreich *)
                    EndFlag:=EoF(Datei);            (* Dateiende erreicht ? *)
                    RestFlag:=(NOT EndFlag);        (* Noch ein Dateirest ? *)
                    DateiAnfang:=AnfangFlag;               (* Dateianfang ? *)
                    AnfangFlag:=Nein;         (* Nach erstem Lesen auf NEIN *)
                                            (* Verwaltereintraege speichern *)
                    WITH Verwalter(.VerwalterAnzahl.) DO BEGIN
                      Nummer:=Nr;
                      Von:=PufferRecord;
                      PufferRecord:=PufferRecord+AnzahlGelesen;
                      Bis:=Pred(PufferRecord);
                    END;
                    Flag:=(Flag AND SchliesseDatei);

                    IF Flag
                      THEN IF PufferRecord=Succ(PufferGroesse)
                        THEN BEGIN         (* Puffer voll --> Puffer leeren *)
                          Flag:=LeerePuffer;
                          WechselFlag:=Ja;
                          IF GleichFlag THEN Lwk1Frage:=Ja;
                          VerwalterAnzahl:=-Ord(EndFlag);
                        END
                        ELSE   (* Puffer noch nicht voll --> weiter fuellen *)
                      ELSE Fehler(DateiName,Ein,4); (* Fehler b. Schliessen *)
                  END
                  ELSE Fehler(DateiName,Ein,2);          (* Blocklesefehler *)
              END
              ELSE Fehler(DateiName,Ein,1)    (* Dateipositionierungsfehler *)
          END
          ELSE Fehler(DateiName,Ein,0);             (* Dateioeffnungsfehler *)
      END;
    FuellePuffer:=Flag;
  END;

BEGIN
  RestFlag:=Nein;                               (* Variablen initialisieren *)
  LeseRecord:=0; SchreibRecord:=0;
  PufferRecord:=0;
  VerwalterAnzahl:=-1;
  GleichFlag:=(Lwk1=Lwk2);                      (* Auf das selbe Laufwerk ? *)
  Lwk1Frage:=(NormLwk(Lwk1) AND NormLwk(Lwk2)); (* Quell- und Ziel-Laufwerk *)
  Lwk2Frage:=NormLwk(Lwk2);                   (* ... = Diskettenlaufwerke ? *)

  (* Solange i<=Zaehler und Flag=TRUE (Kein Fehler) *)
  i:=0; Flag:=Ja;
  WHILE (i<=Zaehler) AND Flag DO BEGIN
    IF Markiert(.i.) THEN Flag:=FuellePuffer(i);
    i:=Succ(i);
  END;

  (* Falls noch ein Rest im Puffer steht *)
  IF Flag AND (VerwalterAnzahl>=0) THEN Flag:=LeerePuffer;

  LoeschDialogZeile; Kopie:=Flag;

  (* Altes Quell-Laufwerk wieder anmelden *)
  IF NormLwk(Lwk1) AND NormLwk(Lwk2)
    THEN QuellFrage
    ELSE SetzeLaufwerk(Lwk1,Nein);

END;
